#ifndef __POINT_HPP__
#define __POINT_HPP__

struct point {
	point() {}
	point(int a,int b) : x(a), y(b) {}
	int x,y;

	int modulo() const {
		return x*x+y*y;
	}

	point operator+(const point& p) const {
		return point(x+p.x,y+p.y);
	}
	point operator-(const point& p) const {
		return point(x-p.x,y-p.y);
	}
	point operator*(int v) const {
		return point(x*v,y*v);
	}
	bool operator!=(const point& p) const {
		return x!=p.x or y!=p.y;
	}
	bool operator==(const point& p) const {
		return x==p.x and y==p.y;
	}
};

int dist_euclidean(const point& p1, const point& p2) {
	return (p1-p2).modulo();
}
int dist_center(const point& p1, const point& p2) {
	return max(std::abs(p1.x-p2.x),std::abs(p1.y-p2.y));
}
int dist_manhatan(const point& p1, const point& p2) {
	return std::abs(p1.x-p2.x)+std::abs(p1.y-p2.y);
}

bool adjacente(const point& p1, const point& p2) {
	return std::abs(p1.x-p2.x)<=1 and std::abs(p1.y-p2.y)<=1 and p1!=p2;
}

#endif
